/*
program:   nls_util.cmd
type:      Object REXX, REXXSAA 6.0 (all platforms)
purpose:   implements functions and classes for easier dealing with NLS-support
version:   2.0
date:      1997-04-15
changed:   1997-06-26, rgf, changed a bug in NLS' SETENTRY and INIT methods
           1997-07-16, rgf, changed instance method "DUMP" of NLS to output
                       to the .error object.
           1999-01-09, rgf, - added default NLS-upper-/lowercases and collating-strings with
                              the help of OS/2 SysMapCase() and SysGetCollate() for the
                              most important character sets (i.e. 437, 863, 852, 865, 850,
                              857, 861, 819/iso8859-1/latin1, 915/iso8859-5/cyrillic,
                              813/iso8859-7/greek/ecma118);

                            - in addition every NLS-object gets an Alias-attribute and for 
                              every charset-alias an entry is created with the class-object 
                              for easier retrieval; this means that in addition to the
                              combination of "country codepage" now also the alias is
                              allowed in all respective methods and routines

                            - made this module operating-system independent; if either
                              SysMapCase() or SysGetCollate() is missing, then the 
                              819/iso8859-1/latin1/-NLS-object will be used as the
                              default character set

                            - new class method HASENTRY (returns .true or .false), 
                              old version deprecated, renamed hasentryOld() (returns .nil or the NLS-object)

author:    Rony G. Flatscher
           Rony.Flatscher@wu-wien.ac.at
           (Wirtschaftsuniversitaet Wien, University of Economics and Business
           Administration, Vienna/Austria/Europe)

needs:     Is_Util.cmd

usage:     see program :), earnestly, only the procedural wrapper-routines are explained
           here. Either call or require this module and then you are set to use the
           routines.

           The routines nls_upper(), nls_lower(), nls_compare() and nls_collate() are 
           carried out by using the definitions of the default NLS-object which one can 
           query with get_nls_default_object() and which can get set with 
           set_nls_default_object():

           get_nls_default_object()
           ... returns the default NLS-object

           set_nls_default_object( [ country, codepage | alias ] )
           ... sets and returns the default NLS-object:
               country, codepage ... OS/2-only, creates the according NLS-object
               alias ............... sets and returns the appropriate NLS-object, valid
                                     pre-defined aliases are:

                                     "437 ibm437 437us"                                       
                                     "863 ibm863 863cafre"                                    
                                     "852 ibm852 852cz"                                       
                                     "865 ibm865 865de"                                       
                                     "850 ibm850 850de"                                       
                                     "857 ibm857 857tk"                                       
                                     "861 ibm861 861is"                                       
                                     "819 ibm819 iso8859-1 iso_8859-1 iso88591 latin1"        
                                     "915 ibm915 iso8859-5 iso_8859-5 iso88595 cyrillic"      
                                     "813 ibm813 iso8859-7 iso_8859-7 iso88597 ecma118 greek" 

           nls_upper( string )
           ... returns the uppercased "string" according to the default NLS-object-definitions

           nls_lower( string )                  
           ... returns the lowercased "string" according to the default NLS-object-definitions

           nls_collate( string )
           ... returns the collated "string" according to the default NLS-object-definitions

           nls_compare( string1, string2 )                
           ... returns "-1" for "string1 < string2", 
                        "0" for "string1 = string2",
                        "1" for "string1 > string2" according to the default NLS-object 
                        collating sequence


comments:  prepared for the "8th International Rexx Symposium 1997", April 1997, Heidelberg/Germany

All rights reserved and copyrighted 1997, 1999 by the author,
no guarantee that it works without errors, etc. etc.

You are granted the right to use this module under the condition that you don't charge money for this module (as you didn't write
it in the first place) or modules directly derived from this module, that you document the original author (to give appropriate
credit) with the original name of the module and that you make the unaltered, original source-code of this module available on
demand.  If that holds, you may even bundle this module (either in source or compiled form) with commercial software.

If you find an error, please send me the description (preferably a *very* short example);
I'll try to fix it and re-release it to the net.
*/


.local ~ nls.Default.Alias = "iso8859-1"        /* use latin1 as default charset on non-OS/2-systems;
                                                   on OS/2 the system's default charset will be used       */

CALL Init_Predefined          /* create NLS-objects with predefined NLS-strings       */

        /* test whether "native" (via RexxUtil) NLS-support is there    */
SIGNAL ON SYNTAX 
CALL SysGetCollate "0", "0"             /* use default contry and codepage      */
CALL SysMapCase    "test", "0", "0"     /* use default contry and codepage      */

continue :                         
    SIGNAL OFF SYNTAX

/* create the default NLS-object, using default country and actual codepage */
IF .nls ~ NLS_SysFunctions THEN
   CALL set_nls_default_object          /* create default NLS-object (country=0, codepage=0)    */
ELSE
   CALL set_nls_default_object .nls.Default.Alias     

EXIT

SYNTAX:
   .nls ~ NLS_SysFunctions = .false     /* indicate not all NLS-Sys()-functions available */
   SIGNAL continue



:: REQUIRES is_util.cmd         /* for IsA() routine    */

:: ROUTINE GET_NLS_DEFAULT_OBJECT       PUBLIC  /* return the default NLS-object */
   RETURN .nls ~ default_NLS

:: ROUTINE SET_NLS_DEFAULT_OBJECT       PUBLIC  /* set the default NLS-object   */
   USE ARG country, codepage

   IF IsA( ARG( 1 ), .nls ) THEN        /* first argument not a country, but a NLS-object?*/
   DO
      .nls ~ default_NLS = ARG( 1 )     /* if so, use it as the default object          */
   END

   ELSE IF ARG() = 1 THEN               /* a codepage-alias in hand     */
   DO
      tmp = .nls ~ entry( arg( 1 ) )    /* try to retrieve NLS-object for given alias   */
      IF tmp <> .nil THEN .nls ~ default_NLS = tmp      /* use existing nls-object      */
                     ELSE 
                     DO                 /* raise syntax-error, missing alias-entry */
                        SIGNAL ON SYNTAX                /* activate exception trapping  */
                        RAISE SYNTAX 40.904 array("SET_NLS_DEFAULT_OBJECT()", "1",,
                                                  "of the valid codepage aliases", arg(1) )
                     END
   END

   ELSE                                 /* no argument or country and codepage given */
   DO

      IF \VAR( "country" ) | country = .nil | "COUNTRY" = country
      THEN country = 0                  /* use default country  */
      ELSE country = STRIP( country )

      IF \VAR( "codepage" ) | codepage = .nil | "CODEPAGE" = codepage
      THEN codepage = 0                 /* use default codepage */
      ELSE codepage = STRIP( codepage )

      tmp = .nls ~ entry( country codepage )           /* try to retrieve NLS-object   */
      IF tmp = .nil THEN 
      DO
         IF .nls ~ NLS_SysFunctions THEN        
            .nls ~ default_NLS = .nls ~ new( country , codePage ) /* create new object */
         ELSE 
            .nls ~ default_NLS = .nls ~ entry( .nls.Default.Alias ) /* set to pre-defined default */
      END
      ELSE .nls ~ default_NLS = tmp       /* use existing NLS-object      */
   END

   RETURN .nls ~ default_NLS

SYNTAX :
   RAISE PROPAGATE                      /* create error in caller       */


   /* procedural wrapper for the most important features of NLS, using the default NLS-object */
:: ROUTINE NLS_UPPER                    PUBLIC          /* do a NLS uppercase */
   USE ARG string
   RETURN .nls ~ default_NLS ~ nls_upper( string )


:: ROUTINE NLS_LOWER                    PUBLIC          /* do a NLS lowercase */
   USE ARG string
   RETURN .nls ~ default_NLS ~ nls_lower( string )


:: ROUTINE NLS_COMPARE                  PUBLIC          /* do a NLS compare */
   USE ARG string1, string2
   RETURN .nls ~ default_NLS ~ nls_compare( string1, string2 )


:: ROUTINE NLS_COLLATE                  PUBLIC          /* translate string into NLS collate */
   USE ARG string
   RETURN .nls ~ default_NLS ~ nls_collate( string )






/* ========================================================================= */
:: CLASS NLS            PUBLIC

/* -------------------------------- class methods ------------------------- */
:: METHOD INIT CLASS
   EXPOSE nlsDirectory default_NLS NLS_SysFunctions

   nlsDirectory = .directory ~ new      /* directory to contain NLS-objects     */
   default_NLS  = .nil                  /* set default NLS-object to .nil       */
   NLS_SysFunctions = .true             /* indicate NLS-Sys()-Functions available */


:: METHOD default_NLS CLASS ATTRIBUTE   /* may contain an NLS-object            */
:: METHOD NLS_SysFunctions CLASS ATTRIBUTE      /* indicates presence of both "SysMapCase()" and "SysGetCollate()"      */


:: METHOD supplier CLASS                /* return a supplier of NLS-objects created so far      */
   EXPOSE nlsDirectory

   RETURN nlsDirectory ~ supplier


:: METHOD entry CLASS                   /* look and return NLS object, if available */
   EXPOSE nlsDirectory

   IF ARG() = 1 THEN tmpKey = ARG( 1 )  /* use given alias for lookup   */

   ELSE                                 /* using "country codepage" for identification  */
   DO
      USE ARG country, codepage
   
      IF \VAR( "country" ) | country = .nil | "COUNTRY" = country
      THEN country = 0
      ELSE country = STRIP( country )
   
      IF \VAR( "codepage" ) | codepage = .nil | "CODEPAGE" = codepage
      THEN codepage = 0
      ELSE codepage = STRIP( codepage )
   
      tmpKey = country codepage
   END

   RETURN nlsDirectory ~ entry( tmpKey )


:: METHOD items    CLASS                /* return number of directory-entries   */
   EXPOSE nlsDirectory
   RETURN nlsDirectory ~ items


:: METHOD hasentry CLASS                /* query for the existence of a specific NLS-object     */
   EXPOSE nlsDirectory

   IF ARG() = 1 THEN tmpKey = ARG( 1 )  /* use given alias for lookup   */
   ELSE                                 /* using "country codepage" for identification  */
   DO
      USE ARG country, codepage
   
      IF \VAR( "country" ) | country = .nil | "COUNTRY" = country
      THEN country = 0
      ELSE country = STRIP( country )
   
      IF \VAR( "codepage" ) | codepage = .nil | "CODEPAGE" = codepage
      THEN codepage = 0
      ELSE codepage = STRIP( codepage )
   
      tmpKey = country codepage
   END

   RETURN nlsDirectory ~ hasentry( tmpKey )     /* returns .true, .false        */


/* deprecated, 99-01-09 */
:: METHOD hasentryOld CLASS                /* query for the existence of a specific NLS-object     */
   EXPOSE nlsDirectory

   IF ARG() = 1 THEN tmpKey = ARG( 1 )  /* use given alias for lookup   */
   ELSE                                 /* using "country codepage" for identification  */
   DO
      USE ARG country, codepage
   
      IF \VAR( "country" ) | country = .nil | "COUNTRY" = country
      THEN country = 0
      ELSE country = STRIP( country )
   
      IF \VAR( "codepage" ) | codepage = .nil | "CODEPAGE" = codepage
      THEN codepage = 0
      ELSE codepage = STRIP( codepage )
   
      tmpKey = country codepage
   END

   RETURN nlsDirectory ~ entry( tmpKey )   /* returns NLS-object, instead of .true/.false !     */



   /* set a new NLS-object, if index already used, set new association, retunr old associated NLS-object */
:: METHOD setentry CLASS                /* set a new NLS-object, if it does not exist yet       */
   EXPOSE nlsDirectory

   IF ARG() = 1 THEN                    /* remove alias-entry, return associated NLS-object     */
   DO
      tmpObject = nlsDirectory ~ entry( ARG(1) )
      nlsDirectory ~ setentry( ARG(1) )
      RETURN tmpObject
   END

   IF IsA( ARG(2), .nls ) THEN          /* second argument a NLS-object, hence set alias/NLS-object-pair    */
   DO
      USE ARG alias, object

      nlsDirectory ~ setentry( alias, object )
      RETURN object
   END

   ELSE                                 /* default processing, expecting country and codepage arguments */
   DO
      USE ARG country, codepage, object
   
      IF \VAR( "country" ) | country = .nil | "COUNTRY" = country
      THEN country = 0
      ELSE country = STRIP( country )
   
      IF \VAR( "codepage" ) | codepage = .nil | "CODEPAGE" = codepage
      THEN codepage = 0
      ELSE codepage = STRIP( codepage )
   
      tmpKey = country codepage         /* create index-value   */
   
      tmpObject = nlsDirectory ~ entry( tmpKey )        /* get NLS-object       */
      IF tmpObject = .nil THEN          /* no entry yet         */
      DO
         IF IsA( object, .nls ) THEN    /* a NLS object given ? */
         DO
            nlsDirectory ~ setentry( tmpKey, object )   /* create new entry     */
            RETURN object
         END
         RETURN .nil                    /* NLS-object to delete is not available*/
      END

      ELSE                              /* NLS-object available         */
      DO
         IF IsA( object, .nls ) THEN    /* a new NLS-object given, hence set it */
            nlsDirectory ~ setentry( tmpKey, object )
         ELSE                           /* remove entry                 */
            nlsDirectory ~ setentry( tmpKey )   
   
         RETURN tmpObject
      END
   END

/* -------------------------------- instance methods ---------------------- */
:: METHOD INIT
   EXPOSE country codepage lowercase uppercase collating_table coll_lower coll_upper aliases

   IF ARG() = 6 THEN                    /* o.k. build from pre-defined strings        */
   DO
      collating_table = .nil            /* only available if NLS-object is created at runtime */
      USE ARG c_cp, aliases, lowercase, uppercase, coll_lower, coll_upper 

      PARSE VAR c_cp country codepage   /* parse into appropriate object-attributes     */

      tmpObj = self ~ class ~ setentry( codepage, country, self )
      tmpAliases = aliases
      DO WHILE tmpAliases <> ""         /* add entry pointing to self (=NLS-object) for each alias      */
         PARSE VAR tmpAliases Alias tmpAliases
         self ~ class ~ setentry( Alias, self )
      END
   END

   ELSE                         /* runtime-creation giving country and codepage */
   DO 
      USE ARG country, codepage

      aliases = ""              /* if created at runtime, no aliases available  */
   
      IF \VAR( "country" )  | country = .nil  | "COUNTRY" = country   THEN
         country = 0
   
      IF \VAR( "codepage" ) | codepage = .nil | "CODEPAGE" = codepage  THEN
         codepage = 0
   
   
      SIGNAL ON SYNTAX     /* build translate to uppercase tables, if wrong combination
                              of country and codepage a syntax error is raised     */
      collating_table = SysGetCollate( country, codepage )
   
      tmpObj = self ~ class ~ setentry( codepage, country, self )
   
      IF tmpObj <> self THEN       /* already set-up, return       */
      DO
         RETURN tmpObj
      END
   
   
      /* determine differing characters for plain lower/uppercase translation */
      lowercase = ""
      uppercase = ""
   
      all_chars = XRANGE( "0"x, "ff"x)     /* create all 256 Bytes */
   
      tmp_up = SysMapCase( all_chars, country, codepage )         /* build translate to uppercase tables */
      DO i = 1 TO LENGTH( all_chars )
         low   = SUBSTR( all_chars, i, 1 )
         upper = SUBSTR( tmp_up,    i, 1 )
   
         IF low <> upper THEN
         DO
            lowercase = lowercase || low
            uppercase = uppercase || upper
         END
      END
   
   
      /* determine differing characters for collating sequence */
      coll_lower = ""
      coll_upper = ""
   
   
      DO i = 1 TO LENGTH( all_chars )
         low   = SUBSTR( all_chars,       i, 1 )
         upper = SUBSTR( collating_table, i, 1 )
   
         IF low <> upper THEN
         DO
            coll_lower = coll_lower || low
            coll_upper = coll_upper || upper
         END
      END
   END


   RETURN self

        /* illegal combination of country and codepage in SysGetCollate() */
SYNTAX : RAISE PROPAGATE        /* raise SYNTAX error in caller */


:: METHOD country                       ATTRIBUTE
:: METHOD codepage                      ATTRIBUTE
:: METHOD aliases                       ATTRIBUTE
:: METHOD coll_lower                    ATTRIBUTE
:: METHOD coll_upper                    ATTRIBUTE
:: METHOD collating_table               ATTRIBUTE
:: METHOD lowercase                     ATTRIBUTE
:: METHOD uppercase                     ATTRIBUTE

:: METHOD makestring                    /* a string representation of a NLS-object      */
   EXPOSE country codepage aliases

   RETURN "a" pp( self ~ class ~ id ) "country=" || pp( country ) || ", codepage=" || pp(codepage )  || ", aliases=" || pp( aliases )


:: METHOD nls_upper                     /* no a NLS uppercase */
   EXPOSE uppercase lowercase
   USE ARG string

   RETURN TRANSLATE( string, uppercase, lowercase )


:: METHOD nls_lower                     /* do a NLS lowercase */
   EXPOSE uppercase lowercase
   USE ARG string

   RETURN TRANSLATE( string, lowercase, uppercase )


:: METHOD nls_compare                   /* do a case sensitive NLS-compare */
   EXPOSE country codepage
   USE ARG string1, string2

        /*  uses Collating Sequence (SysNationalLanguageCompare() is relatively slow, because of external function call)

            string1 < string2 ... -1
            string1 = string2 ...  0
            string1 > string2 ...  1
        */

    string1C = self ~ nls_collate( string1 )
    string2C = self ~ nls_collate( string2 )

    IF string1C < string2C THEN RETURN -1
    IF string1C = string2C THEN RETURN  0
                           ELSE RETURN  1

/*
   RETURN SysNationalLanguageCompare( string1, string2, country, codepage )
*/


:: METHOD nls_collate                   /* translate string with collating table */
   EXPOSE coll_upper coll_lower
   USE ARG string

   RETURN TRANSLATE( string, coll_upper, coll_lower )



:: METHOD dump                          /* show effects on lower/uppercase and collating sequence       */
   EXPOSE aliases country codepage lowercase uppercase coll_lower coll_upper

    .error ~ LINEOUT(  "NLS-object:" )
    .error ~ LINEOUT(  "   country:" pp( country ) "codepage:" pp( codepage ) )
    IF aliases <> "" THEN
       .error ~ LINEOUT(  "   aliases:" pp( aliases ) )
    .error ~ LINEOUT() 
    .error ~ LINEOUT(  "lowercase:" pp( lowercase ) )
    .error ~ LINEOUT(  "uppercase:" pp( uppercase ) )
    .error ~ LINEOUT("")

    .error ~ LINEOUT(  "effect of collating table:" )
    .error ~ LINEOUT("")
   start  = 1
   length = 60
   coll_length = LENGTH( coll_lower )
   DO WHILE start < coll_length
      start = RIGHT( start, 3)
      end   = RIGHT( MIN( coll_length, start + length - 1 ), 3 )
       .error ~ LINEOUT(  " low ("start"-"end"):" pp( STRIP( SUBSTR( coll_lower, start, length ) ) ) )
       .error ~ LINEOUT(  "  up ("start"-"end"):" pp( STRIP( SUBSTR( coll_upper, start, length ) ) ) )
       .error ~ LINEOUT("")
      start = start + length
   END
   RETURN

PP : PROCEDURE                          /* cheap :) pretty-print        */
   RETURN "[" || ARG( 1 ) || "]"







:: ROUTINE Init_Predefined    /* defines and creates NLS-objects */

/* -------------------------------- automatically created - begin ------ */
/* created: [ 19990109 20:24:57 ]
        on: [ OBJREXX 6.00 22 Oct 1998 ]
   version: [ OS/2 2.40 ]
      with: [ Z:\conversi\mk_nls_up.cmd ] */

i         = 0 /* set counter to 0 */

i         = i + 1 
c_cp.i    = "001 437"
aliases.i = "437 ibm437 437us"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 91 93 94 95 96 97 98 A0 A1 A2 A3 A4"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 9A 45 41 8E 41 8F 80 45 45 45 49 49 49 92 4F 99 4F 55 55 59 41 49 4F 55 A5"x

collLow.i = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A8 AD AE AF E1"x
collUp.i  = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 43 55 45 41 41 41 41 43 45 45 45 49 49 49 41 41 45 41 41 4F 4F 4F 55 55 59 4F 55 24 24 24 24 24 41 49 4F 55 4E 4E 3F 21 22 22 53"x

i         = i + 1 
c_cp.i    = "002 863"
aliases.i = "863 ibm863 863cafre"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 80 81 82 83 84 85 87 88 89 8A 8B 8C 8E 90 91 92 93 94 95 96 97 99 9A 9D 9E A2 A3 A8"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 43 55 45 41 41 41 43 45 45 45 49 49 41 45 45 45 4F 45 49 55 55 4F 55 55 55 4F 55 49"x

collLow.i = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 80 81 82 83 84 85 87 88 89 8A 8B 8C 8E 90 91 92 93 94 95 96 97 99 9A 9D 9E A2 A3 A8"x
collUp.i  = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 43 55 45 41 41 41 43 45 45 45 49 49 41 45 45 45 4F 45 49 55 55 4F 55 55 55 4F 55 49"x


i         = i + 1 
c_cp.i    = "042 852"
aliases.i = "852 ibm852 852cz"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 81 82 83 84 85 86 87 88 89 8B 8C 92 93 94 96 98 9C 9F A0 A1 A2 A3 A5 A7 A9 AB AD BE C7 D0 D4 D8 E4 E5 E7 EA EC EE FB FD"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 9A 90 B6 8E DE 8F 80 9D D3 8A D7 91 E2 99 95 97 9B AC B5 D6 E0 E9 A4 A6 A8 8D B8 BD C6 D1 D2 B7 E3 D5 E6 E8 ED DD EB FC"x

collLow.i = "42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE FF"x
collUp.i  = "47 48 4C 4F 54 55 56 57 5A 5B 5C 60 61 64 69 6A 6B 6E 73 76 7B 7C 7D 7E 80 84 85 86 87 88 89 41 47 48 4C 4F 54 55 56 57 5A 5B 5C 60 61 64 69 6A 6B 6E 73 76 7B 7C 7D 7E 80 8A 8B 8C 8D 8E 4B 79 50 46 43 78 49 4B 5F 53 68 68 59 82 43 49 50 5D 5D 66 67 5E 5E 72 72 67 79 74 74 5F 8F 4A 42 58 65 77 44 44 81 81 52 52 BD 82 4A 71 90 91 92 93 94 95 96 42 46 51 71 97 98 99 9A 83 83 9B 9C 9D 9E 9F A0 A1 45 45 A2 A3 A4 A5 A6 A7 A8 A9 4D 4D 4E 53 4E 62 58 59 51 AA AB AC AD 75 78 AE 65 6F 66 63 63 62 70 70 6C 77 6C 7A 7F 7F 75 AF B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA 7A 6D 6D BB BC"x


i         = i + 1 
c_cp.i    = "045 865"
aliases.i = "865 ibm865 865de"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 91 93 94 95 96 97 98 9B A0 A1 A2 A3 A4"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 9A 90 41 8E 41 8F 80 45 45 45 49 49 49 92 4F 99 4F 55 55 59 9D 41 49 4F 55 A5"x

collLow.i = "5B 5C 5D 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 AD AE AF E1"x
collUp.i  = "28 2F 29 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 28 2F 29 43 59 45 41 5B 41 5D 43 45 45 45 49 49 49 5B 5D 45 5B 5B 4F 5C 4F 55 55 59 5C 59 5C 24 5C 24 24 41 49 4F 55 4E 4E 41 4F 3F 21 22 24 53"x


i         = i + 1 
c_cp.i    = "049 850"
aliases.i = "850 ibm850 850de"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 91 93 94 95 96 97 9B A0 A1 A2 A3 A4 C6 D0 E4 E7 EC"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 9A 90 B6 8E B7 8F 80 D2 D3 D4 D8 D7 DE 92 E2 99 E3 EA EB 9D B5 D6 E0 E9 A5 C7 D1 E5 E8 ED"x

collLow.i = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9F A0 A1 A2 A3 A4 A5 A8 AD AE AF B5 B6 B7 BD BE C6 C7 CF D0 D1 D2 D3 D4 D5 D6 D7 D8 DE E0 E1 E2 E3 E4 E5 E7 E9 EA EB EC ED"x
collUp.i  = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 43 55 45 41 41 41 41 43 45 45 45 49 49 49 41 41 45 41 41 4F 4F 4F 55 55 59 4F 55 4F 24 4F 24 41 49 4F 55 4E 4E 3F 21 22 22 41 41 41 24 24 41 41 24 44 44 45 45 45 49 49 49 49 49 4F 53 4F 4F 4F 4F E8 55 55 55 59 59"x


i         = i + 1 
c_cp.i    = "090 857"
aliases.i = "857 ibm857 857tk"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 91 93 94 95 96 97 9B 9F A0 A1 A2 A3 A4 A7 C6 E4 EC"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 9A 90 B6 8E B7 8F 80 D2 D3 D4 D8 D7 98 92 E2 99 E3 EA EB 9D 9E B5 D6 E0 E9 A5 A6 C7 E5 DE"x

collLow.i = "21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F 30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F 40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE FF"x
collUp.i  = "28 36 4C 46 4D 4B 35 39 3A 49 4E 25 24 2D 2C 5B 5C 5D 5E 5F 60 61 62 63 64 27 26 52 53 54 2A 43 65 66 67 69 6A 6B 6C 6E 6F 71 72 73 74 75 76 78 79 7A 7B 7D 7E 80 81 82 83 84 3B 4A 3C 30 21 2F 65 66 67 69 6A 6B 6C 6E 70 71 72 73 74 75 76 78 79 7A 7B 7D 7E 80 81 82 83 84 3D 56 3E 32 A2 68 7F 6A 65 65 65 65 68 6A 6A 6A 70 70 6F 65 65 6A 65 65 76 77 76 7E 7E 70 77 7F 76 47 76 7C 7C 65 70 76 7E 75 75 6D 6D 2B 42 55 5B 5B 29 37 38 85 86 87 88 89 65 65 65 41 8A 8B 8C 8D 45 48 8E 8F 90 91 92 93 94 65 65 95 96 97 98 99 9A 9B 44 76 65 6A 6A 6A 5A 70 70 70 9C 9D 9E 9F 57 70 A0 76 7B 76 76 76 76 59 5A 51 7E 7E 7E 70 83 2A 2E 23 4F 5A 5B 40 3F 50 34 58 31 33 5C 5E 5D A1 5A"x


i         = i + 1 
c_cp.i    = "354 861"
aliases.i = "861 ibm861 861is"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 81 82 83 84 85 86 87 88 89 8A 8C 91 93 94 95 96 98 9B A0 A1 A2 A3"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 9A 90 41 8E 41 8F 80 45 45 45 8B 92 4F 99 8D 55 97 9D A4 A5 A6 A7"x

collLow.i = "42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 AD AE AF E1"x
collUp.i  = "43 44 45 47 49 4A 4B 4C 4E 4F 50 51 52 53 55 56 57 58 59 5A 5C 5D 5E 5F 61 28 2F 29 31 41 43 44 45 47 49 4A 4B 4C 4E 4F 50 51 52 53 55 56 57 58 59 5A 5C 5D 5E 5F 61 28 2F 29 44 5B 48 42 42 42 42 44 48 48 48 46 46 62 42 42 48 63 63 54 64 62 5B 60 60 64 5B 54 24 54 24 24 42 4D 54 5B 42 4D 54 5B 3F 21 22 24 53"x


i         = i + 1 
c_cp.i    = "001 819"
aliases.i = "819 ibm819 iso8859-1 iso_8859-1 iso88591 latin1"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 9A 9C 9E E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F8 F9 FA FB FC FD FE FF"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 8A 8C 8E C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D2 D3 D4 D5 D6 D8 D9 DA DB DC DD DE 9F"x

collLow.i = "20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F 30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F 40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE FF"x
collUp.i  = "43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 90 91 92 93 94 95 96 97 98 99 53 54 55 56 57 58 59 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 5A 5B 5C 5D 5E 5F 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 60 61 62 63 20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F 30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F 40 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 9A 9A 9A 9A 9A 9A 9A 9C 9E 9E 9E 9E A2 A2 A2 A2 9D A7 A8 A8 A8 A8 A8 41 A8 AE AE AE AE B2 B4 AC 9A 9A 9A 9A 9A 9A 9A 9C 9E 9E 9E 9E A2 A2 A2 A2 9D A7 A8 A8 A8 A8 A8 42 A8 AE AE AE AE B2 B4 B2"x


i         = i + 1 
c_cp.i    = "001 915"
aliases.i = "915 ibm915 iso8859-5 iso_8859-5 iso88595 cyrillic"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FE FF"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AE AF"x

collLow.i = "20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F 30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F 40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE FF"x
collUp.i  = "41 48 50 5E 59 5F 5C 4F 51 52 5A 60 45 44 4B 4A 66 67 68 69 6A 6B 6C 6D 6E 6F 47 46 61 62 63 49 58 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 53 5B 54 4D 42 4C 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 55 64 56 4E 20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F 30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F 40 65 92 90 8F 91 95 97 98 9A 9D A0 A7 A6 43 A9 AE 8A 8B 8C 8D 8E 91 93 94 96 99 9B 9C 9E 9F A1 A2 A3 A4 A5 A8 AA AB AC AD AF B0 B1 B2 B3 B4 B5 B6 8A 8B 8C 8D 8E 91 93 94 96 99 9B 9C 9E 9F A1 A2 A3 A4 A5 A8 AA AB AC AD AF B0 B1 B2 B3 B4 B5 B6 5D 92 90 8F 91 95 97 98 9A 9D A0 A7 A6 57 A9 AE"x


i         = i + 1 
c_cp.i    = "001 813"
aliases.i = "813 ibm813 iso8859-7 iso_8859-7 iso88597 ecma118 greek"

mapLow.i  = "61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A C0 DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE"x
mapUp.i   = "41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A BA B6 B8 B9 BA BE C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D3 D3 D4 D5 D6 D7 D8 D9 DA DB BC BE BF"x

collLow.i = "20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F 30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F 40 41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50 51 52 53 54 55 56 57 58 59 5A 5B 5C 5D 5E 5F 60 61 62 63 64 65 66 67 68 69 6A 6B 6C 6D 6E 6F 70 71 72 73 74 75 76 77 78 79 7A 7B 7C 7D 7E 7F 80 81 82 83 84 85 86 87 88 89 8A 8B 8C 8D 8E 8F 90 91 92 93 94 95 96 97 98 99 9A 9B 9C 9D 9E 9F A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 AA AB AC AD AE AF B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 BA BB BC BD BE BF C0 C1 C2 C3 C4 C5 C6 C7 C8 C9 CA CB CC CD CE CF D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA FB FC FD FE FF"x
collUp.i  = "3E 46 54 64 5F 65 63 51 57 58 61 66 43 41 49 48 70 72 73 75 77 78 79 7A 7B 7C 45 44 68 69 6A 47 5E C2 C4 C6 C8 CA CC CE D0 D2 D4 D6 D8 DA DC DE E0 E2 E4 E6 E8 EA EC EE F0 F2 F4 59 62 5A 4C 3F 4B C3 C5 C7 C9 CB CD CF D1 D3 D5 D7 D9 DB DD DF E1 E3 E5 E7 E9 EB ED EF F1 F3 F5 5B 6C 5C 4F 20 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 3E 6F 52 53 60 3E 3E 6D 15 4D 5D 3E 55 6B 40 3E 42 6E 67 74 76 4A 4E 7F 50 89 8F 95 56 A6 71 B3 C0 99 7D 81 83 85 87 8B 8D 91 93 9A 9C 9E A0 A2 A4 A8 AA 3E AC AF B1 B8 BA BC BE 97 B5 80 8A 90 96 B7 7E 82 84 86 88 8C 8E 92 94 9B 9D 9F A1 A3 A5 A9 AB AE AD B0 B2 B9 BB BD BF 98 B6 A7 B4 C1 3E"x

c_cp.0    = i /* save number of codepages */
aliases.0 = i /* save number of codepages */
/* -------------------------------- automatically created - end -------- */


DO i = 1 TO c_cp.0      /* iterate and create NLS-objects       */
   .nls ~ new( c_cp.i, aliases.i, mapLow.i, mapUp.i, collLow.i, collUp.i )
END

RETURN

:: ROUTINE pp; RETURN "[" || ARG( 1 ) || "]"

